# LibVLCSharp documentation

LibVLCSharp is a cross-platform audio and video API for .NET platforms based on [VideoLAN](https://www.videolan.org)'s LibVLC Library. It provides a comprehensive multimedia API that can be used across mobile, server and desktop to render video and output audio. Mono, .NET Framework and .NET Core runtimes are supported.

- [LibVLCSharp](#LibVLCSharp-documentation)
  - [Getting started](#Getting-started)
  - [Installation steps](#Installation-steps).
    - [LibVLC](#LibVLC)
    - [LibVLCSharp](#LibVLCSharp)
    - [Preview versions](libvlc_preview.md)
  - [Migrating from Vlc.Dotnet](migrating_from_Vlc.DotNet.md)
  - [How do I do X?](#How-do-I-do-X?)
  - [Troubleshooting](troubleshooting.md)
  - [MediaPlayerElement](#MediaPlayerElement)
  - [General usage guidelines and best practices](#General-usage-guidelines-and-best-practices)
  - [LibVLC documentation](#libvlc-documentation)
  - [Tutorials](#tutorials)
  - [Community projects](made_with_libvlcsharp.md)
  - [Get Help](#get-help)
  - [Contribute](#contribute)
  - [Resources](#resources)

# Getting started

Follow our [Getting Started](getting_started.md) guide.

# Installation steps

Stable builds can be pulled from nuget.org, while pre-release builds are on feedz.io.

### LibVLC

You may download them from [NuGet](https://www.nuget.org/profiles/videolan).

Please read and follow https://code.videolan.org/videolan/libvlc-nuget.

### LibVLCSharp

- [LibVLCSharp](../LibVLCSharp/README.md)
- [LibVLCSharp.Forms](../LibVLCSharp.Forms/README.md)
- [LibVLCSharp.WPF](../LibVLCSharp.WPF/README.md)
- [LibVLCSharp.GTK](../LibVLCSharp.GTK/README.md)
- [LibVLCSharp.Forms.Platforms.WPF](../LibVLCSharp.Forms.Platforms.WPF/README.md)
- [LibVLCSharp.Forms.Platforms.GTK](../LibVLCSharp.Forms.Platforms.GTK/README.md)
- [LibVLCSharp.WinForms](../LibVLCSharp.WinForms/README.md)
- [LibVLCSharp.Uno](../LibVLCSharp.Uno/README.md)

For Android specific things, head over to [Android](android.md).

### Stable versions

LibVLC 3.x and LibVLCSharp 3.x versions are the current _stable_ libvlc and libvlcsharp versions. Stable builds are recommended for production releases.

Please read and follow https://code.videolan.org/videolan/libvlc-nuget.

### Preview versions

LibVLC 4.x and LibVLCSharp 4.x versions are the current _preview_ libvlc and libvlcsharp versions. Be aware, these builds may be unstable and APIs may change at any time.

LibVLCSharp 3.x nightly builds are also available through feedz.

You may download them from feedz.io. For more details, see [LibVLC preview](libvlc_preview.md).

# How do I do X?

Check out our [How Do I Do X](how_do_I_do_X.md) page.

# MediaPlayerElement

For Xamarin.Forms and Uno platforms, we provide a ready-to-use integrated video control.

Learn what it can do and our future plans for it [here](media_player_element.md).

# General usage guidelines and best practices

[Best practices](best_practices.md)

# LibVLC documentation

Head over to [LibVLC documentation](libvlc_documentation.md).

# Tutorials

Our [tutorials page](tutorials.md) contains references to videos, blogposts and samples.

# Get Help

Post on the [developers LibVLC section](https://forum.videolan.org/viewforum.php?f=32&sid=eada3e1eeccfb3c98b656dfb5b212cce) of the VideoLAN forum.

Post on [StackOverflow](https://stackoverflow.com/questions/tagged/libvlcsharp).

Chat on our [Discord](https://discord.gg/3h3K3JF) or open a [GitLab issue](https://code.videolan.org/videolan/LibVLCSharp/issues).

# Contribute

Contribute to the VideoLAN non-profit https://www.videolan.org/contribute.html

# Resources

Browse the [VideoLAN Wiki](https://wiki.videolan.org/Main_Page/).

Browse the [opensource code](https://code.videolan.org/explore/projects/starred) from VideoLAN.